@@ -13,6 +13,7 @@
 import posixpath
 import os
 import codecs
+import traceback
 try:
     from hashlib import sha1 as sha
 except ImportError:
@@ -25,11 +26,14 @@ from sphinx.errors import SphinxError
 from sphinx.util.osutil import ensuredir, ENOENT, EPIPE
 from sphinx.util.compat import Directive
 
-from nwdiag_sphinxhelper import command, diagparser, builder, DiagramDraw
+from nwdiag_sphinxhelper import command, parser, builder, drawer
 from nwdiag_sphinxhelper import collections, FontMap
 from nwdiag_sphinxhelper import nwdiag, NwdiagDirective
 namedtuple = collections.namedtuple
 
+import blockdiag_sphinxhelper
+detectfont = blockdiag_sphinxhelper.command.detectfont
+
 
 class NwdiagError(SphinxError):
     category = 'Nwdiag error'
@@ -39,7 +43,7 @@ class Nwdiag(NwdiagDirective):
     def run(self):
         try:
             return super(Nwdiag, self).run()
-        except diagparser.ParseException, e:
+        except parser.ParseException, e:
             if self.content:
                 msg = '[%s] ParseError: %s\n%s' % (self.name, e, "\n".join(self.content))
             else:
@@ -56,10 +60,10 @@ def get_image_filename(self, code, format, options, prefix='nwdiag'):
     """
     Get path of output file.
     """
-    if format not in ('PNG', 'PDF', 'SVG'):
+    if format.upper() not in ('PNG', 'PDF', 'SVG'):
         raise NwdiagError('nwdiag error:\nunknown format: %s\n' % format)
 
-    if format == 'PDF':
+    if format.upper() == 'PDF':
         try:
             import reportlab
         except ImportError:
@@ -107,7 +111,7 @@ def get_fontmap(self):
 
         if fontpath:
             config = namedtuple('Config', 'font')(fontpath)
-            _fontpath = command.detectfont(config)
+            _fontpath = detectfont(config)
             fontmap.set_default_font(_fontpath)
     except:
         attrname = '_nwdiag_fontpath_warned'
@@ -127,13 +131,16 @@ def create_nwdiag(self, code, format, filename, options, prefix='nwdiag'):
     draw = None
     fontmap = get_fontmap(self)
     try:
-        tree = diagparser.parse(diagparser.tokenize(code))
+        tree = parser.parse_string(code)
         screen = builder.ScreenNodeBuilder.build(tree)
 
         antialias = self.builder.config.nwdiag_antialias
-        draw = DiagramDraw.DiagramDraw(format, screen, filename,
-                                       fontmap=fontmap, antialias=antialias)
+        draw = drawer.DiagramDraw(format, screen, filename,
+                                  fontmap=fontmap, antialias=antialias)
     except Exception, e:
+        if self.builder.config.nwdiag_debug:
+            traceback.print_exc()
+
         raise NwdiagError('nwdiag error:\n%s\n' % e)
 
     return draw
@@ -193,7 +200,6 @@ def render_dot_html(self, node, code, options, prefix='nwdiag',
             thumb_size = (options['maxwidth'], image_size[1] * ratio)
             if not os.path.isfile(toutfn):
                 image.filename = toutfn
-                image.draw()
                 image.save(thumb_size)
 
     except UnicodeEncodeError, e:
@@ -212,7 +218,7 @@ def render_dot_html(self, node, code, options, prefix='nwdiag',
         if alt is None:
             alt = node.get('alt', self.encode(code).strip())
 
-        if format == 'SVG':
+        if format.upper() == 'SVG':
             tagfunc = make_svgtag
         else:
             tagfunc = make_imgtag
@@ -252,7 +258,7 @@ def latex_visit_nwdiag(self, node):
 
 
 def on_doctree_resolved(self, doctree, docname):
-    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex'):
+    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex', 'epub'):
         return
 
     for node in doctree.traverse(nwdiag):
@@ -280,6 +286,7 @@ def setup(app):
     app.add_config_value('nwdiag_fontpath', None, 'html')
     app.add_config_value('nwdiag_fontmap', None, 'html')
     app.add_config_value('nwdiag_antialias', False, 'html')
+    app.add_config_value('nwdiag_debug', False, 'html')
     app.add_config_value('nwdiag_html_image_format', 'PNG', 'html')
     app.add_config_value('nwdiag_tex_image_format', 'PNG', 'html')
     app.connect("doctree-resolved", on_doctree_resolved)
